/**
 * \file plot.cpp
 * \date November 23, 2011
 * \author Edlira Nano
 */

#include "plot.h"
#include "../../lib/quanti_items/quantiItemBase.h"

Plot::Plot(QWidget *parent) :
    QCustomPlot(parent) {

    qDebug() << "Plot(QWidget *parent) begin";

    xAxis->setLabel("retention time (seconds)");
    yAxis->setLabel("intensity");
    setInteractions(QCP::iRangeDrag | QCP::iRangeZoom );
    axisRects().at(0)->setRangeDrag(Qt::Horizontal);
    axisRects().at(0)->setRangeZoom(Qt::Horizontal);
    legend->setVisible(false);
    legend->setFont(QFont("Helvetica",9));
// set locale to english, so we get english decimal separator:
//setLocale(QLocale(QLocale::English, QLocale::UnitedKingdom));
    setFocusPolicy(Qt::ClickFocus);
    qDebug() << "Plot(QWidget *parent) end";
}

Plot::~Plot() {
    this->clear();
}

void Plot::viewNewPlot(const TreatmentBoxXicExtract * p_treatment) {
    qDebug() << "Plot::viewNewPlot begin";
    this->clear();
    qDebug() << "Created new xic plot";
    _p_xic = p_treatment->getCurrentXic();
    if (_p_xic != 0) {
        QCPGraph * p_graph = this->addGraph();
        p_graph->setName("raw xic");
        QPen pen;
        pen.setColor(getNewColors());
        graph()->setPen(pen);

        _xicPlots.insert(
            std::pair<const TreatmentBox*, QCPGraph *>(p_treatment,
                    p_graph));
        //graph()->setName(lineNames.at(i-QCPGraph::lsNone));
        graph()->setLineStyle(QCPGraph::LineStyle::lsLine);
        p_graph->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssDisc, 2.0));

        for (const pappso::XicElement & element: *_p_xic) {
            graph()->addData(element.rt, element.intensity);
        }
        graph()->rescaleAxes(true);
        this->rescaleAxes();
        replot();
    }

    qDebug() << "Plot::viewNewPlot end";
}
void Plot::updatedCurrentPlot(const TreatmentBoxXicFilter * p_treatment) {
    qDebug() << "Add/update xic plot";
    pappso::NoConstXicSp p_xic = p_treatment->getCurrentXic();
    if (p_xic != 0) {
        QCPGraph * p_graph;
        if (_xicPlots.count(p_treatment) > 0) {
            p_graph = _xicPlots.find(p_treatment)->second;
        } else {
            p_graph = this->addGraph();
            p_graph->setName("filter");
            QPen pen;
            pen.setColor(getNewColors());
            p_graph->setPen(pen);
            _xicPlots.insert(
                std::pair<const TreatmentBox*, QCPGraph *>(p_treatment,
                        p_graph));
        }

        p_graph->setLineStyle(QCPGraph::LineStyle::lsLine);
        p_graph->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssDisc, 2.0));
        for (const pappso::XicElement & element: *p_xic) {
            p_graph->addData(element.rt, element.intensity);
        }

        this->replot();
    }
}
void Plot::updatedPeaks(const TreatmentBoxXicDetect * p_treatment) {
    vector< pappso::XicPeak > p_peaks = p_treatment->getAllXicPeak();

    for (QCPGraph * p_graph_peak:_graph_peak_list) {
        this->removeGraph(p_graph_peak);
    }

    if (p_peaks.size() > 0) {
        //_p_graph_peak_border = this->addGraph();


        QCPGraph * p_graph;
        std::map<const TreatmentBox *, QCPGraph *>::iterator it = _xicPlots.find(p_treatment);
        if (it != _xicPlots.end()) {
            p_graph = it->second;
        } else {
            p_graph = this->addGraph();
            p_graph->setName("detected peaks");
            QPen pen;
            pen.setColor(getNewColors());
            graph()->setPen(pen);
            _xicPlots.insert(
                std::pair<const TreatmentBox*, QCPGraph *>(p_treatment,
                        p_graph));
        }
        p_graph->setLineStyle(QCPGraph::LineStyle::lsNone);
        p_graph->setScatterStyle(QCPScatterStyle::ssCross);
        //p_graph->setBrush(QBrush(QColor(110, 170, 110, 30)));
        QVector<double> intensity;
        QVector<double> rt;

        /*
         // add the group velocity tracer (green circle):
        QCPItemTracer *groupTracer = new QCPItemTracer(customPlot);
        groupTracer->setGraph(graph);
        groupTracer->setGraphKey(5.5);
        groupTracer->setInterpolating(true);
        groupTracer->setStyle(QCPItemTracer::tsCircle);
        groupTracer->setPen(QPen(Qt::green));
        groupTracer->setBrush(Qt::green);
        groupTracer->setSize(7);

         * */
        pappso::Xic xic(*_p_xic);
        std::vector<pappso::XicPeak>::const_iterator itp;
        for (itp = p_peaks.begin(); itp != p_peaks.end(); ++itp) {
            intensity.push_back( itp->getMaxXicElement().intensity);
            rt.push_back( itp->getMaxXicElement().rt);

            QVector<double> intensity_peak;
            QVector<double> rt_peak;
            for (auto & xic_element :xic) {
                if ((xic_element.rt >= itp->getLeftBoundary().rt) && (xic_element.rt <= itp->getRightBoundary().rt)) {
                    intensity_peak.push_back( xic_element.intensity);
                    rt_peak.push_back( xic_element.rt);
                }
            }
            _graph_peak_list.push_back( this->addGraph());
            graph()->setData(rt_peak, intensity_peak);
            graph()->setChannelFillGraph(0);
            graph()->setLineStyle(QCPGraph::LineStyle::lsLine);
            graph()->setScatterStyle(QCPScatterStyle::ssDot);
            graph()->setBrush(QBrush(QColor(170, 110, 110, 30)));
        }
        p_graph->setData(rt, intensity);
        /*
        rt.clear();
        intensity.clear();
        pappso::Xic::const_iterator itx;
        for (itx = xic.begin(); itx != xic.end(); ++itx) {
            intensity.push_back( itx->intensity);
            rt.push_back( itx->rt);
        }
        _p_graph_peak_border->setData(rt, intensity);


        p_graph->setChannelFillGraph(_p_graph_peak_border);
        */
        this->replot();

        /*

        unsigned int plotsize(p_peaks.size());
        mcq_double x1[plotsize], y1[plotsize];
        std::vector<pappso::XicPeak>::const_iterator it;
        int i = 0;
        for (it = p_peaks.begin(); it != p_peaks.end(); ++it) {
            y1[i] = it->getMaxXicElement().intensity;
            x1[i] = it->getMaxXicElement().rt;
            i++;
        }
        curve->setSamples(x1, y1, plotsize);

        curve->attach(this);
        this->replot();
        initZoomer();
        */
    }

}

void Plot::remove(TreatmentBox * box) {
    if (_xicPlots.count(box) > 0) {
        QCPGraph * graph = _xicPlots.find(box)->second;
        removeGraph (graph);

        _xicPlots.erase(_xicPlots.find(box));
        this->replot();
    }
}

void Plot::clear() {
    _xicPlots.clear();
    for( int g=0; g < graphCount(); g++ )
    {
        graph(g)->data()->clear();
    }
    int n= clearPlottables();
    replot();


    qDebug() << "Plot::clear end";
}

void Plot::clearPlot() {
    this->clear();
}

const QColor Plot::getNewColors() {
    QColor color;
    switch (_xicPlots.size()) {
    case 0:
        return (QColor(Qt::black));
    case 1:
        return (QColor(Qt::blue));
    case 2:
        return (QColor(Qt::red));
    case 3:
        return (QColor(Qt::green));
    case 4:
        return (QColor(Qt::yellow));
    default:
        return (QColor(Qt::black));
    }
    return color;
}

void Plot::keyPressEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Control) {
        _control_key = true;
    }
    qDebug() << "Plot::keyPressEvent end";
}

void Plot::keyReleaseEvent(QKeyEvent *event)
{
    if(event->key() == Qt::Key_Control) {
        _control_key = false;
    }
    qDebug() << "Plot::keyReleaseEvent end";
}

void Plot::mousePressEvent(QMouseEvent *event) {
    qDebug() << "Plot::mousePressEvent begin " << xAxis->pixelToCoord(event->x()) << " " << yAxis->pixelToCoord(event->y());
    _click = true;
    _old_x = event->x();
    _old_y = yAxis->pixelToCoord(event->y());
    if (_old_y < 0)  _old_y =0;
    qDebug() << "Plot::mousePressEvent end";
}
void Plot::mouseReleaseEvent(QMouseEvent *event) {
    qDebug() << "Plot::mouseReleaseEvent begin " << xAxis->pixelToCoord(event->x()) << " " << yAxis->pixelToCoord(event->y());
    _click = false;
    qDebug() << "Plot::mouseReleaseEvent end";
}
void Plot::mouseMoveEvent (QMouseEvent *event) {
    if (_click) {
        qDebug() << "Plot::mouseMoveEvent begin " << xAxis->pixelToCoord(event->x()) << " " << yAxis->pixelToCoord(event->y());
        pappso::pappso_double y = yAxis->pixelToCoord(event->y());
        if (y < 0) {
            y =0;
        }
        if (_control_key) {
            if (y > 0) {
                this->yAxis->scaleRange( _old_y/y, 0);
            }
        }
        else {
            this->xAxis->moveRange(xAxis->pixelToCoord(_old_x)-xAxis->pixelToCoord(event->x()));

        }
        _old_x = event->x();
        _old_y = y;
        replot();
        qDebug() << "Plot::mouseMoveEvent end";
    }
}
